#!/bin/bash

export COMPONENT_HASH=253bd0543800735a0a07cff388086301cb61d106
PKG_VERSION=1.1+cdh6.3.0
FULL_VERSION=1.1-cdh6.3.0
BASE_VERSION=1.1
PKG_RELEASE=1279813
CDH_CUSTOMER_PATCH=p0
CDH_PARCEL_CUSTOM_VERSION=1.1+cdh6.3.0-1279813.xenial
